%% How do we build the complex dft matrix? What structure does this matrix have?
% build function dft.m same as the real dft matrix we see in the slides
% from lecture but now we are using the complex exponentials so we get the
% same output format as calling fft
% we can visualize the dft matrix to make sure things look right
clear all; close all;

N=64;
dftmat = dft(N);

%what happens if I plot the real part of the second column what about imag
%(plot these on the same axes)
x = 0:N-1
plot(x, real(dftmat(:,2)), 'r')
hold on;
plot(x, imag(dftmat(:,2)), 'g')

figure;
plot(x, real(dftmat(:,3)), 'r')
hold on;
plot(x, imag(dftmat(:,3)), 'g')

%these are the sines and cosines from the dft matrix we saw in class except
%now we obtained them from the combined complex exponential! 

%Now we'll use this and compare to the fft in matlab in one example:


%% Creating, decomposing, and re-creating signals
clear all; close all; 

% let's create a signal that is composed of 3 sinusoids

% To create a sinusoid in matlab, you can use this formula:
% A*cos(omega*n - phi)

% A = amplitude
% omega = frequency. can be further characterized as:
%   (2*pi*k)/N
%   k = # of cycles
%   N = total number of samples
% n = sample position. Ranges from 0 to N-1
% psi = phase shift. Ranges from 0 to 2*pi (then repeats)

% using this formula, let's make sinusoids!

% we'll have 64 samples
N = 64;
n = [0:N-1];

% random positive number for amplitude

amp1 = randn(1); 
amp2 = randn(1);
amp3 = randn(1);

% for phase, we want a number between 0 and 2pi. 
% here's how to do that in matlab:
minVal = 0;
maxVal = 2*pi;
psi1 = (maxVal-minVal).*rand(1) + minVal;
psi2 = (maxVal-minVal).*rand(1) + minVal;
psi3 = (maxVal-minVal).*rand(1) + minVal;

% to make the frequencies, let's be less random. We'll make frequencies
% corresponding to k=1, k=3, k=5.
k1 = 1;
k2 = 3;
k3 = 5;

% put them all together!
sin1 = amp1*cos((2*pi*k1)/N*n - psi1);
sin2 = amp2*cos((2*pi*k2)/N*n - psi2);
sin3 = amp3*cos((2*pi*k3)/N*n - psi3);

% let's see what we made
figure(1)
subplot(1,2,1)
title('the signals we made')
hold on
plot(n,sin1,'r') % sin1 in red
plot(n,sin2,'b') % sin2 in blue
plot(n,sin3,'g') % sin3 in green

% notice how there is 1 complete cycle for the red curve, 3 complete cycles
% for the blue curve, and 5 complete cycles for the green curve,
% corresponding to their respective k values

% Now we'll add them all up to make our weird signal
summedSin = sin1 + sin2 + sin3;
summedSin = summedSin'; % make column vec
% add that to our plot!
plot(n,summedSin,'k', 'LineWidth',2) % summedSin in black

%% weird looking, right?!

% Now we will use fourier transform to see what frequencies make up our new
% weird signal

fftOutput = fft(summedSin);
th = 1e-8;
fftOutput(abs(fftOutput) < th) = 0; % because of numerics, make all small magnitude transform values actually 0. 
% ^ this creates a N dimensional vector of complex numbers.
% fftOutput(1) corresponds to k=0; fftOutput(2) corresponds to k=1... all
% the way to fftOutput(N) corresponds to k=N-1
% The real part of each number tells you the amplitude of the signal at
% each value of k


amps = abs(fftOutput)*(2/N); 
%dont worry about the 2/N this is a scale factor that just gets the 
%amplitudes the same
%remember this is not the same as real / imag.. they are two different
%coordinate systems
% the imaginary part of each number tells you the phase of the signal at
% each value of k
phases = angle(fftOutput);

% Now in lecture we saw there were a couple ways to plot these spectrum.
% First plot the amplitude and phase from k = 0 to k = N/2
% so let's see what we have
f = [0:N/2];
figure
subplot(2,1,1)
stem(f,amps(1:N/2+1))
title('amplitudes')

subplot(2,1,2)
stem(f,phases(1:N/2+1))
title('phases')
%% What do you notice?




%Let's use our dft function and try the same thing: 
dftmat  = dft(N);
fftOutput2 = dftmat*summedSin;
fftOutput2(abs(fftOutput2) < th) = 0;

amps2 = abs(fftOutput2)*(2/N);
phases2 = angle(fftOutput2);
f = [0:N/2];
figure
subplot(2,1,1)
stem(f,amps2(1:N/2+1))
title('amplitudes')

subplot(2,1,2)
stem(f,phases2(1:N/2+1))
title('phases')

%These give you the same answer! 

%% Second way involves shifting the fft to be centered around 0 
figure;
fftOutputshift = fftshift(fftOutput)
fshift = (-N/2:N/2-1)
stem(fshift,abs(fftOutputshift)*(2/N))
title('Shifted Amplitudes')

% Get non-zero components of the amplitude
amps = round(amps,6);
nonZeroAmplitudes = find(amps>0);

% but that's in matlab indexing. let's convert to math indexing
nonZeroAmplitudes_math = n(nonZeroAmplitudes)

% extract the amplitudes and cosines
freq1 = nonZeroAmplitudes_math(1);
freq2 = nonZeroAmplitudes_math(2);
freq3 = nonZeroAmplitudes_math(3);

newAmp1 = amps(nonZeroAmplitudes(1));
newAmp2 = amps(nonZeroAmplitudes(2));
newAmp3 = amps(nonZeroAmplitudes(3));

%% Multiply by -1 becuase the phase output from FFT is always a negative 
% version of the actual phase.
newPsi1 = -phases(nonZeroAmplitudes(1));
newPsi2 = -phases(nonZeroAmplitudes(2));
newPsi3 = -phases(nonZeroAmplitudes(3));

% re-create the signals
newSin1 = newAmp1*cos((2*pi*freq1)/N*n - newPsi1);
newSin2 = newAmp2*cos((2*pi*freq2)/N*n - newPsi2);
newSin3 = newAmp3*cos((2*pi*freq3)/N*n - newPsi3);

figure(1)
subplot(1,2,2)
title('the signals we re-created')
hold on
plot(n, newSin1,'r-')
plot(n, newSin2,'b-')
plot(n, newSin3,'g-')
plot(n, newSin1+newSin2+newSin3, 'k-', 'LineWidth',2)



%% Now let's explore some of the key aspects of fourier analysis
% F(impulse response) = Frequency Response ... what does this mean?
% (Theory / on the board)

%% Convolution Theorem- what is it useful for? 
% Let's say we have some signal that is being corrupted by low-frequency
% noise
clear all;close all;
load lab6-noisysound.mat
%Fs=2999;
%Ts=1/Fs
%t=[0:Ts:1];
%F_A=440; %Frequency of note A is 440 Hz
%F_C = 554.37
%F_E = 329.63
%noise = cos(2*pi*60*t);
%A=cos(2*pi*F_A*t) + cos(2*pi*F_C*t)+ cos(2*pi*F_E*t) + noise;
%A=A';
N= length(A)
sound(A,Fs);

% Let's plot the spectrum:
dftmat = dft(N);
fftsound = dftmat*A*(2/N);
fftsound = fft(A)*(2/N);
f = Fs*(0:(N/2))/N;
stem(f, abs(fftsound(1:N/2+1)))

%% We see the note at 440Hz but then we see this 60Hz noise. We want to
% convolve the signal with a high-pass filter.. but we can use the
% convolution theorem to define the filter in the frequency domain and do a
% pointwise multiplication
figure;
filt = zeros(3000,1);
filt(300:800) = 1;

fftout = filt.*fftsound; 

stem(f,abs(fftout(1:N/2+1)))

%Now we can do the ifft to get back the tone with the 60hz noise removed
%using the conjugate transpose of the dft matrix. We can also use ifft -
% try both!
Adenoise = real(dftmat'*fftout);
%Adenoise = real(ifft(fftout))*(N/2);
sound(Adenoise,Fs);






